/*
  datetime.c

  Copyright (c) Parallax Inc 2015. All rights MIT licensed;
                see end of file.
*/  

#include "simpletools.h"
#include "datetime.h"

static int stack[174 + 64];
static volatile int dtcog;
volatile int dtlock;
volatile datetime *dt_sys_datetime;

void secondctr(void *par);

int dt_start(datetime *dts)
{
  dt_sys_datetime = dts;
  dts->etv = dte_toETV(dts->y, dts->mo, dts->d, dts->h, dts->m, dts->s);
  dtcog = 1 + cogstart(secondctr, dts, stack, sizeof(stack));
  dtlock = locknew();
  if(dtlock >= 0) return dtcog; else return -dtcog;
}

void dt_stop()
{
  if(dtcog)
  {
    cogstop(dtcog);
    dtcog = 0;
  }
  lockret(dtlock);
}

void secondctr(void *par)
{
  datetime *dts = (datetime *) par;
  int dNow;
  int tNow;
  int t = CNT;
  int dt = CLKFREQ;
  while(1)
  {
    waitcnt(t+=dt);
    while(lockset(dtlock));
    dts->etv++;
    
    dNow = dte_dateETV(dts->etv);
    tNow = dte_timeETV(dts->etv);

    dts->y = dNow >> 16;
    dts->mo = (dNow >> 8) & 0xFF;
    dts->d = dNow & 0xFF;
    dts->h = tNow >> 16;
    dts->m = (tNow >> 8) & 0xFF;
    dts->s = tNow & 0xFF;
    lockclr(dtlock);
  }
}

/*
  TERMS OF USE: MIT License
 
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:
 
  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.
 
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

