/*
  libdatetime.c

  Test harness for the datetime library

  Copyright (c) Parallax Inc 2015. All rights MIT licensed;
                see end of file.
*/  

#include "simpletools.h"
#include "datetime.h"

//              y    mo  d   h  m   s
datetime dts = {2015, 9, 25, 2, 39, 35};      // dts -> date, time (system) 
datetime dta, dtt;                            // Alarm and temp dates, times

int main()
{
  // Set system time from pre-initialized date
  dt_start(&dts);
  for(int i = 0; i < 5; i++)
  {
    printi("%04d/%02d/%02d, %02d:%02d:%02d\n",
          dts.y, dts.mo, dts.d, dts.h, dts.m, dts.s);
    pause(2000);
  }

  // Change system time by creating loading values into a a temporary 
  // time from the terminal and then copy to dts.
  printi("\nEnter Time\n");
  printi("y: ");
  scan("%d", &dtt.y);
  printi("mo: ");
  scan("%d", &dtt.mo);
  printi("d: ");
  scan("%d", &dtt.d);
  printi("h: ");
  scan("%d", &dtt.h);
  printi("m: ");
  scan("%d", &dtt.m);
  printi("s: ");
  scan("%d", &dtt.s);
  dt_copy(&dts, &dtt);
  
  for(int i = 0; i < 5; i++)
  {
    printi("%04d/%02d/%02d, %02d:%02d:%02d\n",
          dts.y, dts.mo, dts.d, dts.h, dts.m, dts.s);
    pause(2000);
  }
  
  // Change system date & time with dt_set
  dt_set(&dts, 2015, 12, 31, 23, 59, 55);

  // Set alarm and test
  dta = (datetime) {2016, 1, 1, 0, 0, 0};
  printi("\nTarget Time\n");
  printi("%04d/%02d/%02d, %02d:%02d:%02d\n",
        dts.y, dts.mo, dts.d, dts.h, dts.m, dts.s);
  
  int secDiff, nymsg = 0;

  for(int i = 0; i < 7; i++)
  {
    printi("%04d/%02d/%02d, %02d:%02d:%02d\n",
          dts.y, dts.mo, dts.d, dts.h, dts.m, dts.s);
    secDiff = dt_cmpSec(&dta, &dts);
    printi("secDif = %d\n", secDiff);
    if(secDiff <= 0 && nymsg == 0)
    {
      print("\nHappy New Year!!!\n\n");
      nymsg = 1;
    }      
    pause(2000);
  }
  
  // Add offset to time
  print("\nAdd 1 d, 12 h, 30 m, 40 s to dts system time\n");
  dt_copy(&dtt, &dts);
  dt_addSec(&dtt, 40 * SECONDS);
  dt_addSec(&dtt, 30 * MINUTES);
  dt_addSec(&dtt, 12 * HOURS);
  dt_addSec(&dtt, 1 * DAYS);
  dt_copy(&dts, &dtt);
  
  for(int i = 0; i < 5; i++)
  {
    printi("%04d/%02d/%02d, %02d:%02d:%02d\n",
          dts.y, dts.mo, dts.d, dts.h, dts.m, dts.s);
    pause(2000);
  }
  print("\nDone!\n");
}    

/*
  TERMS OF USE: MIT License
 
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:
 
  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.
 
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

